/**
 * Represents a financial position in a portfolio.
 */
export class Position {
    /**
     * The price of the last trade made for this position.
     */
    readonly lastTrade: number;

    /**
     * The date and time of the last trade made for this position.
     */
    readonly lastTradeTime: Date;

    /**
     * The change in the position value.
     */
    readonly change: number;

    /**
     * The percentage change in the position value.
     */
    readonly changePercent: number;

    /**
     * The earnings per share of the position.
     */
    readonly earningsPerShare: number;

    /**
     * The market capitalization of the position.
     */
    readonly marketCap: number;

    /**
     * The symbol of the position.
     */
    readonly symbol: string;

    /**
     * Represents a position in a portfolio.
     * @constructor
     * @param {number} lastTrade - The price of the last trade made for this position.
     * @param {Date} lastTradeTime - The date and time of the last trade made for this position.
     * @param {number} change - The change in the position value.
     * @param {number} changePercent - The percentage change in the position value.
     * @param {number} earningsPerShare - The earnings per share of the position.
     * @param {number} marketCap - The market capitalization of the position.
     * @param {string} symbol - The symbol of the position.
     */
    constructor(
        lastTrade: number,
        lastTradeTime: Date,
        change: number,
        changePercent: number,
        earningsPerShare: number,
        marketCap: number,
        symbol: string
    ) {
        this.lastTrade = lastTrade;
        this.lastTradeTime = lastTradeTime;
        this.change = change;
        this.changePercent = changePercent;
        this.earningsPerShare = earningsPerShare;
        this.marketCap = marketCap;
        this.symbol = symbol;
    }
}

/**
 * Represents a portfolio of financial positions.
 */
export class Portfolio {
    /**
     * An array of positions held in the portfolio.
     */
    readonly positions: Position[];

    /**
     * Creates a new Portfolio instance.
     * @constructor
     * @param {Position[]} positions - An array of Position objects representing the positions in the portfolio.
     */
    constructor(positions: Position[]) {
        this.positions = positions;
    }
}

/**
 * A provider for fetching a portfolio.
 */
export interface PortfolioProvider {

    /**
     * Fetches the portfolio.
     * @returns A promise that resolves with the fetched portfolio.
     */
    readonly fetchPortfolio: () => Promise<Portfolio>;
}
